// EngraveSetup.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "MachDevice.h"
#include "probeconfig.h"
#include "XMLProfile.h"
#include "MenuHandler.h"
#include "Mach4View.h"
#include "Engine.h"
#include "TrajectoryControl.h"
#include ".\probeconfig.h"

extern TrajectoryControl *MainPlanner;
extern CMach4View *MachView;
extern TrajBuffer *Engine;

double m_ImageX;
double m_ImageY;
double m_StepsX;
double m_StepsY;
int    m_TriggerLen;
double m_YStepOver;
double m_length;
double m_width;
double m_feedrate;
bool   m_40MS;
bool   m_Inverse;
bool   m_Aspect;

extern double estartx;
extern double estarty;
extern bool edir;

extern EngraveImage import;

//Monitor thread

EngraveSetup *monitor = NULL;
extern EngraveThread* pThread;

IMPLEMENT_DYNCREATE(EngraveThread, CWinThread)

EngraveThread::EngraveThread(  )
{

}

EngraveThread::~EngraveThread()
{
	if( monitor != NULL )
	{
		monitor->CloseWindow();
		monitor = NULL;
	}
}

BOOL EngraveThread::InitInstance()
{
	if( monitor != NULL ) return false;
	monitor = new EngraveSetup();
	monitor->DoModal();
	delete monitor;
	monitor = NULL;
	return TRUE;
}

int EngraveThread::ExitInstance()
{
	pThread = NULL;
	return CWinThread::ExitInstance();
}

BEGIN_MESSAGE_MAP(EngraveThread, CWinThread)
END_MESSAGE_MAP()

extern CXMLProfile *DevProf;
extern MenuHandler *Menus;

// EngraveSetup dialog

IMPLEMENT_DYNAMIC(EngraveSetup, CDialog)
EngraveSetup::EngraveSetup(CWnd* pParent /*=NULL*/)
	: CDialog(EngraveSetup::IDD, pParent)
	
//	, m_40MS(FALSE), m_Inverse(FALSE), m_Aspect(FALSE)		//??????
{
	Loaded = false;
}

EngraveSetup::~EngraveSetup()
{
}

void EngraveSetup::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_LENGTH, m_length);
	DDX_Text(pDX, IDC_WIDTH, m_width);
	DDX_Text(pDX, IDC_EFEEDRATE, m_feedrate);
	DDX_Control(pDX, IDC_PICTURE, m_Pic);
	DDX_Control(pDX, IDC_LENGTH, m_LengthBox);
	DDX_Text(pDX, IDC_XSIZE, m_ImageX);
	DDX_Text(pDX, IDC_YSIZE, m_ImageY);
	DDX_Text(pDX, IDC_XPIXEL, m_StepsX);
	DDX_Text(pDX, IDC_YPIXEL, m_StepsY);
	DDX_Text(pDX, IDC_TRIGGERLEN, m_TriggerLen);
	DDX_Text(pDX, IDC_YSTEPOVER, m_YStepOver);
	DDX_Check(pDX, IDC_40MS, m_40MS);
	DDX_Check(pDX, IDC_40MS2, m_Inverse);
	DDX_Check(pDX, IDC_40MS3, m_Aspect);
}

BEGIN_MESSAGE_MAP(EngraveSetup, CDialog)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDC_ECANCEL, OnBnClickedCancel)
	ON_WM_PAINT()
	ON_BN_CLICKED(IDC_CALC, OnBnClickedCalc)
	ON_WM_TIMER()
	ON_WM_DRAWITEM()
	ON_WM_CTLCOLOR()
	ON_EN_CHANGE(IDC_LENGTH, OnEnChangeLength)
	ON_EN_CHANGE(IDC_WIDTH, OnEnChangeWidth)
	ON_EN_CHANGE(IDC_XSIZE, OnEnChangeXsize)
	ON_BN_CLICKED(IDC_40MS, OnBnClicked40ms)
	ON_BN_CLICKED(IDC_40MS2, OnBnClicked40ms2)
	ON_EN_CHANGE(IDC_TRIGGERLEN, OnEnChangeTriggerlen)
	ON_BN_CLICKED(IDC_40MS3, OnBnClicked40ms3)
	ON_EN_CHANGE(IDC_YPIXEL, OnEnChangeYpixel)
END_MESSAGE_MAP()

// EngraveSetup message handlers

BOOL EngraveSetup::OnInitDialog()
{
	m_Type = 0;
	
	CDialog::OnInitDialog();

	m_40MS = Engine->TriggerBits;
	static const TCHAR szFilters[] =
	{
		TEXT("Jpeg\0")                                       TEXT("*.jpg\0")
			TEXT("Bitmap\0")                                     TEXT("*.bmp\0")
			TEXT("Png files\0")                                  TEXT("*.png\0")
	};

	CFileDialog *dlg = new CFileDialog(true, "JPegs","*.jpg", OFN_OVERWRITEPROMPT, _T("Jpg FIles (*.JPG)\0*.jpg\0") ); 
	dlg->m_ofn.lpstrFilter = szFilters;
	if( dlg->DoModal() == IDCANCEL) 
	{
		delete dlg;
		return -1;
	}
	lpszPathName = dlg->GetPathName();
	delete dlg;
	
	OnBnClickedLoadimage();

	if( !Loaded )
	{
		MessageBox( "Image load error!");
		return -1;
	}

	m_ImageX = import.image->GetWidth();
	m_ImageY = import.image->GetHeight();
	
	//convert image to grey scale..
	import.image->GrayScale();
	
	UpdateData(false);		// update the form data first!!!
	
	OnBnClickedCalc();		// then force a recalculation of data values

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void EngraveSetup::OnBnClickedOk()
{
	OnBnClickedCalc();		// force a recalculation of data values

	Engine->TriggerBits = (m_40MS == TRUE);
	
	if (m_width < 0 || m_length < 0)
	{
		AfxMessageBox("ERROR... Width and Length must set be > 0.00");
		return;
	}
	
	if (m_feedrate < 0)
	{
		AfxMessageBox("ERROR... Rate must set be > 0.00");
		return;
	}
	
	if ( m_TriggerLen > 1024000)
	{
		AfxMessageBox("ERROR... Must be fewer than 1 million trigger bits.");
		return;
	}
    
	if ( !Menus->OKtoStart() ) 
		OnOK();
	else
	{
	  Menus->SEQUENCE = SEQ_START;
	  estartx = 0;
	  estarty = 0;
	  edir = true;
	  Menus->SetMachError("Press Cycle Start to begin scanning Seq.");
	  OnOK();
	}
}

void EngraveSetup::OnBnClickedCancel()
{
	OnCancel();
}

void EngraveSetup::OnBnClickedLoadimage()
{
	if( import.LoadImage( lpszPathName ))
	{
		Loaded = true;
	}
	else
		Loaded = false;
}

void EngraveSetup::OnPaint()
{
	CDialog::OnPaint();
	if( Loaded) 
	{
		CRect sz;
		CPaintDC dc( &m_Pic ); // device context for painting
		m_Pic.GetClientRect(&sz);
		import.image->Stretch(dc, sz );
	}
}

void EngraveSetup::OnBnClickedCalc()
{
	// calculate the image run data

	UpdateData(true); //get any new data from the form

	//first, the size of the Y output pixel in steps (m_length). 
	if (m_Aspect)				
		m_length = m_width * (m_ImageY / m_ImageX);	// use width as scaling value 

	m_StepsX = (MainPlanner->StepsPerAxis[0] * m_width) / m_ImageX;
	m_StepsY = (MainPlanner->StepsPerAxis[1] * m_length) / m_ImageY;
    m_TriggerLen = (int)(MainPlanner->StepsPerAxis[0] * m_width);
	
	UpdateData(false);	// update the displayed form data
}

void EngraveSetup::OnTimer(UINT nIDEvent)
{
	// TODO: Add your message handler code here and/or call default
	CDialog::OnTimer(nIDEvent);
}

void EngraveSetup::OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct)
{
	// TODO: Add your message handler code here and/or call default
	 
	CDialog::OnDrawItem(nIDCtl, lpDrawItemStruct);
}

HBRUSH EngraveSetup::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
	HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

	return hbr;
}

void EngraveSetup::OnEnChangeLength()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}

void EngraveSetup::OnEnChangeWidth()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}

void EngraveSetup::OnEnChangeXsize()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}

void EngraveSetup::OnBnClicked40ms()
{
	// TODO: Add your control notification handler code here
	OnBnClickedCalc();
}

void EngraveSetup::OnBnClicked40ms2()
{
	// TODO: Add your control notification handler code here
	OnBnClickedCalc();
}

void EngraveSetup::OnEnChangeTriggerlen()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}

void EngraveSetup::OnBnClicked40ms3()
{
	// TODO: Add your control notification handler code here
	
	OnBnClickedCalc();
}

void EngraveSetup::OnEnChangeYpixel()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}
